import 'dart:async';
import 'package:flutter/widgets.dart';
import 'package:flutter/services.dart';
import 'package:google_applovin_unity_ads/google_applovin_unity_ads.dart';
import '../utils.dart';

enum BannerAdEvent {
  /// Called when an impression is recorded for an ad.
  impression,

  /// Called when an ad request failed.
  loadFailed,

  /// Called when an ad is received.
  loaded,

  /// Called when the ad starts loading
  loading,
}

class BannerSize {
  /// The Size of the Banner.
  final Size size;

  const BannerSize(this.size);

  static const BannerSize SMART_BANNER = BannerSize(Size(-1, -2));

  static const BannerSize ADAPTIVE = BannerSize(Size(-1, -1));

  static const BannerSize FLUID = BannerSize(Size(-3, -4));

  static const BannerSize BANNER = BannerSize(Size(320, 50));

  static const BannerSize LARGE_BANNER = BannerSize(Size(320, 100));

  static const BannerSize MEDIUM_RECTANGLE = BannerSize(Size(320, 250));

  static const BannerSize FULL_BANNER = BannerSize(Size(468, 60));

  static const BannerSize LEADERBOARD = BannerSize(Size(728, 90));

  static const BannerSize WIDE_SKYSCRAPER = BannerSize(Size(160, 600));

  factory BannerSize.fromWH(double width, double height) {
    return BannerSize(Size(width, height));
  }

  @override
  String toString() => '${size.width}x${size.height}';
}

class BannerAdController extends LoadShowAd<BannerAdEvent>
    with AttachableMixin {

  Stream<Map<BannerAdEvent, dynamic>> get onEvent => super.onEvent;

  /// Creates a new native ad controller
  BannerAdController({
    Duration loadTimeout = kDefaultLoadTimeout,
    Duration timeout = kDefaultAdTimeout,
  }) : super(
          loadTimeout: loadTimeout,
          timeout: timeout,
        );

  void init() {
    channel.setMethodCallHandler(_handleMessages);
    GoogleApplovinUnityAds.platform.invokeMethod('call.ads.banner', {'id': id});
  }

  void dispose() {
    super.dispose();
    GoogleApplovinUnityAds.platform.invokeMethod('dispose.ads.banner', {
      'id': id,
    });
    attach(false);
  }

  Future<dynamic> _handleMessages(MethodCall call) async {
    if (isDisposed) return;
    switch (call.method) {
      case 'loading':
        isLoaded = false;
        onEventController.add({BannerAdEvent.loading: null});
        break;
      case 'onAdFailedToLoad':
        isLoaded = false;
        onEventController.add({
          BannerAdEvent.loadFailed: AdError.fromJson(call.arguments),
        });
        break;
      case 'onAdLoaded':
        isLoaded = true;
        onEventController.add({BannerAdEvent.loaded: call.arguments});
        break;
      case 'onAdImpression':
        onEventController.add({BannerAdEvent.impression: null});
        break;
      default:
        break;
    }
  }

  Future<bool> load({
    bool force = false,

    Duration? timeout,
  }) async {
    ensureAdNotDisposed();
    // assertMobileAdsIsInitialized();
    if (!debugCheckAdWillReload(isLoaded, force)) return false;
    isLoaded = (await channel.invokeMethod<bool>('load.ads.banner').timeout(
      timeout ?? loadTimeout,
      onTimeout: () {
        if (!onEventController.isClosed && !isLoaded) {
          onEventController.add({
            BannerAdEvent.loadFailed: AdError.timeoutError,
          });
        }
        return false;
      },
    ))!;
    if (isLoaded) lastLoadedTime = DateTime.now();
    return isLoaded;
  }
}
