import 'dart:async';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

void assertPlatformIsSupported() {
  // Google's AdMOB supports only Android and iOS
  assert(
    Platform.isAndroid || Platform.isIOS,
    'The current platform does not support native ads. '
    'The platforms that support it are Android and iOS',
  );
}

bool debugCheckAdWillReload([bool? isLoaded, bool? force]) {
  isLoaded ??= false;
  force ??= false;
  if (isLoaded && !force) {
    print('An ad is already avaiable, no need to load another');
    return false;
  }
  return true;
}

typedef AdBuilder = Widget Function(BuildContext context, Widget child);

class AdError {
  static const AdError timeoutError = AdError(
    code: -1,
    message: 'Timeout achieved',
  );


  final int? code;
  final String? message;

  const AdError({
    required this.code,
    required this.message,
  });

  /// Retrieve an [AdError] from a json
  static AdError fromJson(/* Map<String, dynamic> */ json) {
    return AdError(
      code: json['errorCode'],
      message: json['message'],
    );
  }

  @override
  String toString() => '#$code: $message';
}

mixin UniqueKeyMixin {
  final _key = UniqueKey();

  /// The unique key of [this] class
  String get id => _key.toString();
}

mixin AttachableMixin {
  bool _attached = false;

  /// Check if the controller is attached to an Ad
  bool get isAttached => _attached;

  @mustCallSuper
  void attach([bool attach = true]) {
    if (attach) _assertControllerIsNotAttached();
    _attached = attach;
  }

  /// Ensure the controller is not attached
  void _assertControllerIsNotAttached() {
    assert(
      !isAttached,
      'This controller has already been attached to an ad. '
      'You need one controller for each',
    );
  }
}

const Duration kDefaultLoadTimeout = Duration(minutes: 2);
const Duration kDefaultAdTimeout = Duration(minutes: 30);
const bool kDefaultNonPersonalizedAds = false;
const ServerSideVerificationOptions? kServerSideVerification = null;

abstract class LoadShowAd<T> with UniqueKeyMixin {
  @protected
  final onEventController = StreamController<Map<T, dynamic>>.broadcast();

  Stream<Map<T, dynamic>> get onEvent => onEventController.stream;

  // @protected
  late MethodChannel channel;

  bool _loaded = false;

  /// Check if the ad is loaded
  bool get isLoaded => _loaded;

  final Duration timeout;

  @protected
  DateTime? lastLoadedTime;

  bool get isAvailable {
    if (!isLoaded || lastLoadedTime == null) return false;
    final difference = lastLoadedTime!.difference(DateTime.now());
    if (difference > timeout) return false;
    return true;
  }

  @protected
  set isLoaded(bool loaded) {
    _loaded = loaded;
  }

  bool _disposed = false;

  bool get isDisposed => _disposed;

  /// The unit id used on this ad. Can be null
  final String? unitId;

  final Duration loadTimeout;

  final bool nonPersonalizedAds;

  final ServerSideVerificationOptions? serverSideVerificationOptions;

  @mustCallSuper
  LoadShowAd({
    this.unitId,
    this.loadTimeout = kDefaultLoadTimeout,
    this.timeout = kDefaultAdTimeout,
    this.nonPersonalizedAds = kDefaultNonPersonalizedAds,
    this.serverSideVerificationOptions = kServerSideVerification,
  }) {
    channel = MethodChannel(id);
    init();
  }

  @protected
  void init();
  Future<bool> load();
  Future<bool> show() {
    throw UnimplementedError('This was not implemented for this ad');
  }

  @mustCallSuper
  void dispose() {
    ensureAdNotDisposed();
    _disposed = true;
    onEventController.close();
  }

  @protected
  void ensureAdNotDisposed() {
    assert(!_disposed, 'You can NOT use a disposed ad');
  }

  @protected
  void ensureAdAvailable() {
    assert(isAvailable, 'You can only use an available ad.');
  }
}

class ServerSideVerificationOptions {
  /// The user id to be used in server-to-server reward callbacks.
  final String? userId;

  /// The custom data to be used in server-to-server reward callbacks
  final String? customData;

  ServerSideVerificationOptions({this.userId, this.customData});

  Map<String, dynamic> toJson() => {
        'userId': this.userId,
        'customData': this.customData,
      };
}
